import os

url_dc = "https://raw.githubusercontent.com/otmaneelallaki/HAX712X-DOS/main/Project/Prediction/Dataset/DataTrining.csv"
path_target = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "..", "data", "bicycle_db.csv"
)
